import logging
import os
import pwd

from pathlib import Path
from hydra.core.config_search_path import ConfigSearchPath
from hydra.plugins.search_path_plugin import SearchPathPlugin


logger = logging.getLogger(__name__)

current_user = pwd.getpwuid(os.getuid()).pw_name
cwd = Path.cwd()


class UrartuPlugin(SearchPathPlugin):
    def manipulate_search_path(self, search_path: ConfigSearchPath) -> None:
        search_path.append(provider="urartu", path=f"file://{cwd}/configs_{current_user}")
        search_path.append(provider="urartu", path=f"pkg://{cwd}/configs_{current_user}")
        search_path.append(provider="urartu", path=f"file://{cwd}/configs")
        search_path.append(provider="urartu", path=f"pkg://{cwd}/configs")



from typing import Tuple, Union

import tiktoken
from langchain.schema import HumanMessage
from langchain_openai import AzureChatOpenAI
from transformers import AutoModelForCausalLM, AutoTokenizer

from urartu.common.model import Model


class ModelOpenAI(Model):
    def __init__(self, cfg) -> None:
        super().__init__(cfg)

    @property
    def model(self) -> AutoModelForCausalLM:
        if self._model is None:
            self._model = AzureChatOpenAI(
                deployment_name=self.cfg.name,
                openai_api_type=self.cfg.openai_api_type,
                openai_api_version=self.cfg.openai_api_version,
                azure_endpoint=self.cfg.azure_openai_endpoint,
                openai_api_key=self.cfg.azure_openai_api_key,
            )
        return self._model

    def generate(self, prompt: Union[str, Tuple[str, str]], generate_cfg):
        output = self.model(HumanMessage(content=prompt))

        return output

    def _get_num_tokens(self, string: str, encoding_name: str = "gpt-3.5-turbo") -> int:
        encoding = tiktoken.encoding_for_model(encoding_name)
        num_tokens = len(encoding.encode(string))
        return num_tokens



import torch
from transformers import AutoModelForCausalLM, AutoTokenizer

from urartu.common.device import Device
from urartu.common.model import Model
from urartu.utils.dtype import eval_dtype


class ModelCausalLanguage(Model):
    def __init__(self, cfg) -> None:
        super().__init__(cfg)
        self._tokenizer = None

    @property
    def model(self) -> AutoModelForCausalLM:
        if self._model is None:
            self._model = AutoModelForCausalLM.from_pretrained(
                self.cfg.name,
                cache_dir=self.cfg.cache_dir,
                device_map=Device.get_device(),
                torch_dtype=eval_dtype(self.cfg.dtype),
                token=self.cfg.api_token,
            )

            for param in self._model.parameters():
                param.requires_grad = False
            self._model.eval()
        return self._model

    @property
    def tokenizer(self):
        if self._tokenizer is None:
            self._tokenizer = AutoTokenizer.from_pretrained(self.cfg.name)
            self._tokenizer.pad_token = self._tokenizer.eos_token
        return self._tokenizer

    def generate(self, prompt: str, generate_cfg=None):
        if not generate_cfg:
            generate_cfg = self.cfg.get("generate")
        self.model.eval()

        prompt_tokenized = self.tokenizer(prompt, return_tensors="pt", padding=True, truncation=True)
        prompt_tensor = prompt_tokenized["input_ids"].to(self.model.device)
        attention_mask = prompt_tokenized["attention_mask"].to(self.model.device)
        with torch.no_grad():
            output_tokenized = self.model.generate(
                input_ids=prompt_tensor, attention_mask=attention_mask, **generate_cfg
            )
        output = self.tokenizer.decode(output_tokenized[0], skip_special_tokens=True)

        return output
